#!/usr/bin/env python
# Description: generate initial trycrystal geometry for two twin embryo case 
# Change Log:
# 1. Yanqing Su created construct_TwoParaEmbryos.py on Mon June 19, 2019.

usage="""
        Usage: construct_TwoParaEmbryos.py nx ny nz s t ph th om buffer in/vtk   
               nx,ny,nz - periodicity in x, and thickness in y and z
               s - ratio of spacing/thickness
               t - twin embryo thickness (voxels)
               ph,th,om - Euler angle for the neighboring grain
               buffer - thickness for the buffer layer in y and z direction
               in/vtk - output filetype in(input datafile for the Trycrystals.f90)/vtk(vtk datefile)
        Example: construct_TwoParaEmbryos.py 3 510 510 4 6 30 90 60 25 vtk
"""
import os
import re
import sys
import math
import random
import copy
# Default setting
nx = 3
ny = 510
nz = 510
buffer1 = 25

def gen_vtkfile(nx,ny,nz,buffer1,parent_ph,parent_th,parent_om,parent_grain,parent_jph):
  filename  = "input.vtk"
  fout = open(filename,"w")

  fout.write("# vtk DataFile Version 4.0\n")
  fout.write("# \n")
  fout.write("ASCII\n")
  fout.write("DATASET STRUCTURED_POINTS\n")
  fout.write("DIMENSIONS %d %d %d\n"%(nx+1,ny+2*buffer1+1,nz+2*buffer1+1))
  fout.write("ASPECT_RATIO 1 1 1\n")
  fout.write("ORIGIN 0 0 0\n")
  fout.write("CELL_DATA %d\n"%(nx*(ny+2*buffer1)*(nz+2*buffer1)))

  fout.write("SCALARS grain int\n")
  fout.write("LOOKUP_TABLE default\n")

  for k in range(nz+2*buffer1):
    for j in range(ny+2*buffer1):
      for i in range(nx):
#        print parent_ph[i][j][k],parent_th[i][j][k],parent_om[i][j][k]),i,j,k,parent_grain[i][j][k],parent_jph[i][j][k]
        fout.write("     %d\n"%(parent_grain[i][j][k]))
  fout.close()
  return 0


def gen_VPFFT_str(nx,ny,nz,buffer1,parent_ph,parent_th,parent_om,parent_grain,parent_jph):
#  if not os.path.exists(dirname):
#        os.mkdir(dirname)
#        print("Directory " , dirname ,  " Created ")
#  else:    
#        print("Directory " , dirname ,  " already exists")  
  filename = "paralleltwin"
  print filename 
  fout = open(filename,"w")
  for i in range(nx):
    for j in range(ny+2*buffer1):
      for k in range(nz+2*buffer1):
#        print parent_ph[i][j][k],parent_th[i][j][k],parent_om[i][j][k]),i,j,k,parent_grain[i][j][k],parent_jph[i][j][k]
        fout.write(" %10.3f  %10.3f  %10.3f %d %d %d %d %d\n"%(parent_ph[i][j][k],parent_th[i][j][k],parent_om[i][j][k],i+1,j+1,k+1,parent_grain[i][j][k],parent_jph[i][j][k]))
#        fout.write(" %10.3f  %10.3f  %10.3f\n"%(parent_ph[i][j][k],parent_th[i][j][k],parent_om[i][j][k])
 
  print "what"
  fout.close()
  return 0

# Main Program
if len(sys.argv) > 5:
  nx=int(sys.argv[1])
  ny=int(sys.argv[2])
  nz=int(sys.argv[3])
  coefs=float(sys.argv[4])
  twinthick=int(sys.argv[5])
  phn=int(sys.argv[6])
  thn=int(sys.argv[7])
  omn=int(sys.argv[8])
  buffer1=int(sys.argv[9])
  option=sys.argv[10]
 
  php=0.0; thp=0.0; omp=0.0; jgr=1; jph=1
  print nx
  nz1 = nz+2*buffer1; ny1 = ny+2*buffer1
  parent_ph = [[([0] * nz1) for i in range(ny1)] for j in range(nx)]
  parent_th  = copy.deepcopy(parent_ph); parent_om = copy.deepcopy(parent_ph)
  parent_grain = copy.deepcopy(parent_ph); parent_jph = copy.deepcopy(parent_ph)

  twink = - math.tan((45.0/180*math.pi))
#  twinthick = 24 #twin thickness is set to 6 voxels argv[5]
  twins1 = twinthick*coefs
  twins2 = twinthick*(coefs+2)
  z0 = nz1/2; y0 = ny/3+buffer1

  bhigherB = twink*y0-z0 - twins1/2*math.sqrt(1+twink*twink) #(higher bottom plane   z - twinky + blowerT = 0)
  blowerT = twink*y0-z0  + twins1/2*math.sqrt(1+twink*twink) #(lower top palne z - twinky + blowerB = 0) 
  bhigherT = twink*y0-z0 - twins2/2*math.sqrt(1+twink*twink) #(higher top plane   z - twinky + bhigherT = 0)
  blowerB = twink*y0-z0 + twins2/2*math.sqrt(1+twink*twink) #(lower bottom palne z - twinky + blowerB = 0) 

  lenembryo = ny/3/10

  for i in range(nx):
    for j in range(ny1):
      for k in range(nz1):
        if j < buffer1 or j > ny+buffer1-1 or k < buffer1 or k > nz+buffer1-1:
          parent_ph[i][j][k]=random.randint(0,361)
          parent_th[i][j][k]=random.randint(-90,91)
          parent_om[i][j][k]=random.randint(0,361)
          parent_grain[i][j][k]=5; parent_jph[i][j][k]=jph;#buffer zone
        elif j > ny/3+buffer1-1 and j < ny/3*2+buffer1-1:
          parent_ph[i][j][k]=php; parent_th[i][j][k]=thp; parent_om[i][j][k]=omp;
          parent_grain[i][j][k]=1; parent_jph[i][j][k]=jph;#parent zone
          if (((k - twink*j + blowerT) <= 0 and (k - twink*j + blowerB) >= 0) or ((k - twink*j + bhigherT) <= 0 and (k - twink*j + bhigherB) >= 0)) and ( j < ny/3+buffer1-1+lenembryo):
            parent_grain[i][j][k]=3#twin zone
        else:
#          parent_ph[i][j][k]=75; parent_th[i][j][k]=62; parent_om[i][j][k]=0;
          parent_ph[i][j][k]=phn; parent_th[i][j][k]=thn; parent_om[i][j][k]=omn;
          parent_grain[i][j][k]=2; parent_jph[i][j][k]=jph;#neighbour zone


  if option == "vtk":
    gen_vtkfile(nx,ny,nz,buffer1,parent_ph,parent_th,parent_om,parent_grain,parent_jph)
  elif option == "in":
    gen_VPFFT_str(nx,ny,nz,buffer1,parent_ph,parent_th,parent_om,parent_grain,parent_jph)
  else:
    print "Error: wrong number of arguments!!!"
    print usage

  
